@@ -24,8 +24,13 @@
 #ifdef HAVE_OFF_T_64
 /* if off_t is already 64 bits, be happy, and don't worry about the
  * loff_t and llseek stuff */
-#define MT_OFF_T off_t
-#define MT_SIZE_T size_t
+# define MT_OFF_T off_t
+# if SIZEOF_SIZE_T == 4
+/* Some systems (NetBSD) apparently have 64 bit off_t, but 32 bit size_t ... */
+#  define MT_SIZE_T off_t
+# else
+#  define MT_SIZE_T size_t
+# endif
 #endif
 
 #ifndef MT_OFF_T
@@ -84,6 +89,8 @@ typedef struct {
 #define min(a,b) ((a) < (b) ? (a) : (b))
 #define MAX_OFF_T_B(bits) \
 	((((mt_off_t) 1 << min(bits-1, sizeof(mt_off_t)*8 - 2)) -1) << 1 | 1)
+#define MAX_SIZE_T_B(bits) \
+	((((mt_size_t) 1 << min(bits-1, sizeof(mt_size_t)*8 - 2)) -1) << 1 | 1)
 
 #if defined(HAVE_LLSEEK) || defined(HAVE_LSEEK64)
 # define SEEK_BITS 63
@@ -95,21 +102,17 @@ extern const mt_off_t max_off_t_31;
 extern const mt_off_t max_off_t_41;
 extern const mt_off_t max_off_t_seek;
 
-extern off_t truncBytes32(mt_off_t off);
+extern off_t truncBytes32(mt_off_t off); /* truncMtOffToOff */
+extern uint32_t truncMtOffTo32u(mt_off_t off);
+extern uint32_t truncOffTo32u(off_t off);
+extern uint32_t truncSizeTo32u(size_t siz);
 extern int fileTooBig(mt_off_t off);
-mt_off_t sectorsToBytes(Stream_t *This, off_t off);
-
-mt_size_t getfree(Stream_t *Stream);
-int getfreeMinBytes(Stream_t *Stream, mt_size_t ref);
-
-Stream_t *find_device(char drive, int mode, struct device *out_dev,
-		      union bootsector *boot,
-		      char *name, int *media, mt_size_t *maxSize,
-		      int *isRop);
+extern int fileSizeTooBig(mt_size_t off);
+ssize_t truncOffToSsize(off_t off);
+size_t truncOffToSize(off_t off);
 
 int mt_lseek(int fd, mt_off_t where, int whence);
 
-
-unsigned int log_2(int);
+unsigned int log_2(unsigned int);
 
 #endif
