@@ -511,13 +511,36 @@ static int dos_target_lookup(MainParam_t *mp, const char *arg)
 	}
 }
 
+/*
+ * Is target a Unix directory
+ * -1 error occured
+ * 0 regular file
+ * 1 directory
+ */
+static int unix_is_dir(const char *name)
+{
+	struct stat buf;
+	if(stat(name, &buf) < 0)
+		return -1;
+	else
+		return 1 && S_ISDIR(buf.st_mode);
+}
+
 static int unix_target_lookup(MainParam_t *mp, const char *arg)
 {
 	char *ptr;
 	mp->unixTarget = strdup(arg);
 	/* try complete filename */
-	if(access(mp->unixTarget, F_OK) == 0)
+	if(access(mp->unixTarget, F_OK) == 0) {
+		switch(unix_is_dir(mp->unixTarget)) {
+		case -1:
+			return ERROR_ONE;
+		case 0:
+			mp->targetName="";
+			break;
+		}
 		return GOT_ONE;
+	}
 	ptr = strrchr(mp->unixTarget, '/');
 	if(!ptr) {
 		mp->targetName = mp->unixTarget;
@@ -548,6 +571,8 @@ int main_loop(MainParam_t *mp, char **argv, int argc)
 	if(argc != 1 && mp->targetName) {
 		fprintf(stderr,
 			"Several file names given, but last argument (%s) not a directory\n", mp->targetName);
+		FREE(&mp->targetDir);
+		return 1;
 	}
 
 	for (i = 0; i < argc; i++) {
@@ -650,13 +675,9 @@ char *mpBuildUnixFilename(MainParam_t *mp)
 		return 0;
 	strcpy(ret, mp->unixTarget);
 	if(*target) {
-#if 1 /* fix for 'mcopy -n x:file existingfile' -- H. Lermen 980816 */
-		if(!mp->targetName && !mp->targetDir) {
-			struct MT_STAT buf;
-			if (!MT_STAT(ret, &buf) && !S_ISDIR(buf.st_mode))
-				return ret;
-		}
-#endif
+		/* fix for 'mcopy -n x:file existingfile' -- H. Lermen 980816 */
+		if(!mp->targetName && !mp->targetDir && !unix_is_dir(ret))
+			return ret;
 		strcat(ret, "/");
 		if(!strcmp(target, ".")) {
 		  target="DOT";
