@@ -107,7 +107,7 @@ static int _unix_write(MainParam_t *mp, int needfilter, const char *unixFile)
 	Stream_t *File=mp->File;
 	Stream_t *Target, *Source;
 	struct MT_STAT stbuf;
-	int ret;
+	ssize_t ret;
 	char errmsg[80];
 
 	File->Class->get_data(File, &mtime, 0, 0, 0);
@@ -135,11 +135,9 @@ static int _unix_write(MainParam_t *mp, int needfilter, const char *unixFile)
 				}
 				sFd = get_fd(File);
 				if(sFd == -1) {
-					fprintf(stderr, "Not ok Unix file ==> good\n");
-				}
-				if((!MT_FSTAT(sFd, &srcStbuf)) &&
-				   stbuf.st_dev == srcStbuf.st_dev &&
-				   stbuf.st_ino == srcStbuf.st_ino) {
+				} else if((!MT_FSTAT(sFd, &srcStbuf)) &&
+					   stbuf.st_dev == srcStbuf.st_dev &&
+					   stbuf.st_ino == srcStbuf.st_ino) {
 					fprintf(stderr, "Attempt to copy file on itself\n");
 					return ERROR_ONE;
 				}
@@ -289,7 +287,9 @@ static int writeit(struct dos_name_t *dosname,
 {
 	Stream_t *Target;
 	time_t now;
-	int type, fat, ret;
+	int type;
+	ssize_t ret;
+	uint32_t fat;
 	time_t date;
 	mt_size_t filesize, newsize;
 	Arg_t *arg = (Arg_t *) arg0;
@@ -302,7 +302,7 @@ static int writeit(struct dos_name_t *dosname,
 		return -1;
 	}
 
-	if(fileTooBig(filesize)) {
+	if(fileSizeTooBig(filesize)) {
 		fprintf(stderr, "File \"%s\" too big\n", longname);
 		return 1;
 	}
@@ -351,7 +351,7 @@ static int writeit(struct dos_name_t *dosname,
 		fat_free(arg->mp.targetDir, fat);
 		return -1;
 	} else {
-		mk_entry(dosname, arg->attr, fat, truncBytes32(newsize),
+		mk_entry(dosname, arg->attr, fat, (uint32_t)newsize,
 				 now, &entry->dir);
 		return 0;
 	}
@@ -392,7 +392,7 @@ static Stream_t *subDir(Stream_t *parent, const char *filename)
 	direntry_t entry;
 	initializeDirentry(&entry, parent);
 
-	switch(vfat_lookup(&entry, filename, -1, ACCEPT_DIR, 0, 0, 0, 0)) {
+	switch(vfat_lookup_zt(&entry, filename, ACCEPT_DIR, 0, 0, 0, 0)) {
 	    case 0:
 		return OpenFileByDirentry(&entry);
 	    case -1:
@@ -537,6 +537,7 @@ void mcopy(int argc, char **argv, int mtype)
 				break;
 			case 'T':
 				arg.convertCharset = 1;
+				 /*-fallthrough*/
 			case 'a':
 			case 't':
 				arg.textmode = 1;
@@ -558,7 +559,7 @@ void mcopy(int argc, char **argv, int mtype)
 				batchmode = 1;
 				break;
 			case 'o':
-				handle_clash_options(&arg.ch, c);
+				handle_clash_options(&arg.ch, (char) c);
 				break;
 			case 'D':
 				if(handle_clash_options(&arg.ch, *optarg))
@@ -570,6 +571,7 @@ void mcopy(int argc, char **argv, int mtype)
 				usage(1);
 			default:
 				break;
+
 		}
 	}
 
